@@ -6,21 +6,20 @@
  *
  * Copyright (C) 2001 Anders Johansson ajh@atri.curtin.edu.au
  *
- * This file is part of MPlayer.
+ * This file is part of mpv.
  *
- * MPlayer is free software; you can redistribute it and/or modify
+ * mpv is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * MPlayer is distributed in the hope that it will be useful,
+ * mpv is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License along
- * with MPlayer; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * with mpv.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -204,7 +203,6 @@ static int af_open(struct af_instance* af){
 const struct af_info af_info_equalizer = {
   .info = "Equalizer audio filter",
   .name = "equalizer",
-  .flags = AF_FLAGS_NOT_REENTRANT,
   .open = af_open,
   .priv_size = sizeof(af_equalizer_t),
   .options = (const struct m_option[]) {
