@@ -180,9 +180,11 @@ const struct mp_cmd_def mp_cmds[] = {
   { MP_CMD_DROP_BUFFERS, "drop-buffers", },
 
   { MP_CMD_AF, "af", { ARG_STRING, ARG_STRING } },
+  { MP_CMD_AF_COMMAND, "af-command", { ARG_STRING, ARG_STRING, ARG_STRING } },
   { MP_CMD_AO_RELOAD, "ao-reload", },
 
   { MP_CMD_VF, "vf", { ARG_STRING, ARG_STRING } },
+  { MP_CMD_VF_COMMAND, "vf-command", { ARG_STRING, ARG_STRING, ARG_STRING } },
 
   { MP_CMD_VO_CMDLINE, "vo-cmdline", { ARG_STRING } },
 
