@@ -37,6 +37,7 @@ void cr_perror(const wchar_t *prefix)
 int cr_runproc(wchar_t *name, wchar_t *cmdline)
 {
     STARTUPINFO si;
+    STARTUPINFO our_si;
     PROCESS_INFORMATION pi;
     DWORD retval = 1;
 
@@ -47,6 +48,12 @@ int cr_runproc(wchar_t *name, wchar_t *cmdline)
     si.hStdError = GetStdHandle(STD_ERROR_HANDLE);
     si.dwFlags |= STARTF_USESTDHANDLES;
 
+    // Copy the list of inherited CRT file descriptors to the new process
+    our_si.cb = sizeof(our_si);
+    GetStartupInfo(&our_si);
+    si.lpReserved2 = our_si.lpReserved2;
+    si.cbReserved2 = our_si.cbReserved2;
+
     ZeroMemory(&pi, sizeof(pi));
 
     if (!CreateProcessW(name, cmdline, NULL, NULL, TRUE, 0,
