@@ -17,8 +17,12 @@ struct sd {
     const struct sd_functions *driver;
     void *priv;
 
-    struct demuxer *demuxer;
+    struct attachment_list *attachments;
     struct mp_codec_params *codec;
+
+    // Set to false as soon as the decoder discards old subtitle events.
+    // (only needed if sd_functions.accept_packets_in_advance == false)
+    bool preload_ok;
 };
 
 struct sd_functions {
@@ -33,8 +37,8 @@ struct sd_functions {
     bool (*accepts_packet)(struct sd *sd); // implicit default if NULL: true
     int (*control)(struct sd *sd, enum sd_ctrl cmd, void *arg);
 
-    void (*get_bitmaps)(struct sd *sd, struct mp_osd_res dim, double pts,
-                        struct sub_bitmaps *res);
+    void (*get_bitmaps)(struct sd *sd, struct mp_osd_res dim, int format,
+                        double pts, struct sub_bitmaps *res);
     char *(*get_text)(struct sd *sd, double pts);
 };
 
