@@ -1,5 +1,3 @@
-#define COBJMACROS 1
-#define _WIN32_WINNT 0x600
 #include <malloc.h>
 #include <stdlib.h>
 #include <process.h>
