@@ -288,7 +288,7 @@ def build(ctx):
         ( "video/decode/dec_video.c"),
         ( "video/decode/dxva2.c",                "d3d-hwaccel" ),
         ( "video/decode/d3d11va.c",              "d3d-hwaccel" ),
-        ( "video/decode/d3d.c",                  "d3d-hwaccel" ),
+        ( "video/decode/d3d.c",                  "win32" ),
         ( "video/decode/vaapi.c",                "vaapi-hwaccel" ),
         ( "video/decode/vd_lavc.c" ),
         ( "video/decode/videotoolbox.c",         "videotoolbox-hwaccel" ),
@@ -327,7 +327,6 @@ def build(ctx):
         ( "video/out/dither.c" ),
         ( "video/out/filter_kernels.c" ),
         ( "video/out/opengl/angle_dynamic.c",    "egl-angle" ),
-        ( "video/out/opengl/angle_common.c",     "egl-angle" ),
         ( "video/out/opengl/common.c",           "gl" ),
         ( "video/out/opengl/context.c",          "gl" ),
         ( "video/out/opengl/context_angle.c",    "egl-angle" ),
